with Names;
with AocsMgr;
with UnitMgr;

package body ModeMgr.Transitions is

   use Names;

   Trans : Transition_T renames ModeMgr.Trans;

   procedure To_StandbyMode is
      -- Invariant: ModeMgr.Mode = Off
      Ret_Value_from_Call : Boolean;
   begin

      case Trans.Step is

         when 0 =>

            AocsMgr.Select_Controller (AocsMgr.None);

            UnitMgr.Release_All;

            Trans.Step := Trans.Step + 1;

         when 1 =>

            Ret_Value_from_Call := UnitMgr.All_Off;
            if Ret_Value_from_Call then

               ModeMgr.Separation_StartCheck;

               Trans.Done := True;

            end if;

         when others =>

            null;

      end case;

   end To_StandbyMode;

   procedure To_SafeMode is
      -- Invariant: ModeMgr.Separation_Done
      Ret_Value_from_Call : Boolean;
   begin

      case Trans.Step is

         when 0 =>

            AocsMgr.Select_Controller (AocsMgr.Virtual);

            AocsMgr.Stop_PLI_Management;

            UnitMgr.Release_All;

            Trans.Step := Trans.Step + 1;


         when 1 =>

            Ret_Value_from_Call := UnitMgr.All_Off;
            if Ret_Value_from_Call then

               UnitMgr.Start_Mode_Specific_Unit_Conf (Safe);

               Trans.Step := Trans.Step + 1;

            end if;


         when 2 =>

            Ret_Value_from_Call := UnitMgr.Unit_Conf_Done (Mode => Safe);
            if Ret_Value_from_Call then

               AocsMgr.Select_Controller (AocsMgr.Coarse_Pointing_Control);

               AocsMgr.Prepare_Controller;

               Trans.Step := Trans.Step + 1;

            end if;

         when 3 =>

            Ret_Value_from_Call := AocsMgr.Controller_Is_Ready;
            if Ret_Value_from_Call then
               Ret_Value_from_Call := UnitMgr.No_Reconfiguration;
               if Ret_Value_from_Call then
                  -- We should not lock units which are under
                  -- reconfiguration!!

                  AocsMgr.Start_Controller;

                  UnitMgr.Lock_All_Selected (Mode => Safe);

                  Trans.Done := True;

               end if;
            end if;

         when others =>

            null;

      end case;

   end To_SafeMode;


   procedure To_NominalMode is
      Ret_Value_from_Call : Boolean;
   begin

      case Trans.Step is

         when 0 =>

            AocsMgr.Select_Controller (AocsMgr.Virtual);

            AocsMgr.Stop_PLI_Management;

            UnitMgr.Start_Mode_Specific_Unit_Conf (Nominal);

            Trans.Step := Trans.Step + 1;

         when 1 =>

            Ret_Value_from_Call := UnitMgr.Unit_Conf_Done (Mode => Nominal);
            if Ret_Value_from_Call then

               AocsMgr.Select_Controller (AocsMgr.Fine_Pointing_Control);

               AocsMgr.Prepare_Controller;

               Trans.Step := Trans.Step + 1;

            end if;

         when 2 =>

            Ret_Value_from_Call := AocsMgr.Controller_Is_Ready;
            if Ret_Value_from_Call then
               Ret_Value_from_Call := UnitMgr.No_Reconfiguration;
               if Ret_Value_from_Call then

                  AocsMgr.Start_Controller;

                  UnitMgr.Lock_All_Selected (Mode => Nominal);

                  Trans.Done := True;

               end if;
            end if;

         when others =>

            null;

      end case;

   end To_NominalMode;


   procedure To_PrepMode is
      Ret_Value_from_Call : Boolean;
   begin

      case Trans.Step is

         when 0 =>

            AocsMgr.Stop_PLI_Management;

            Trans.Step := Trans.Step + 1;

         when 1 =>

            Ret_Value_from_Call := UnitMgr.No_Reconfiguration;
            if Ret_Value_from_Call then

               Ret_Value_from_Call := UnitMgr.State_by_U_Is (
                  GPS, GPS_Coarse_Navigation);
               if Ret_Value_from_Call then

                  UnitMgr.Start_Transition_U (GPS, GPS_Fine_Navigation);

               end if;

               UnitMgr.Start_Transition_U (PLI, PLI_Standby);

               Trans.Step := Trans.Step + 1;

            end if;

         when 2 =>

            Ret_Value_from_Call := UnitMgr.State_by_U_Is (
               GPS, GPS_Fine_Navigation);
            if Ret_Value_from_Call then
               Ret_Value_from_Call := UnitMgr.State_by_U_Is (
                  PLI, PLI_Standby);
               if Ret_Value_from_Call then

                  UnitMgr.Lock (PLI);

                  AocsMgr.Start_PLI_Management;

                  Trans.Done := True;

               end if;
            end if;

         when others =>

            null;

      end case;

   end To_PrepMode;


   procedure To_ScienceMode is
      Ret_Value_from_Call : Boolean;
   begin

      case Trans.Step is

         when 0 =>

            UnitMgr.Start_Transition_U (PLI, PLI_Science);

            Trans.Step := Trans.Step + 1;

         when 1 =>

            Ret_Value_from_Call := UnitMgr.State_by_U_Is (
               PLI, PLI_Science);
            if Ret_Value_from_Call then

               Trans.Done := True;

            end if;

         when others =>

            null;

      end case;

   end To_ScienceMode;


   procedure Back_To_Nominal is
      Ret_Value_from_Call : Boolean;
   begin

      case Trans.Step is

         when 0 =>

            AocsMgr.Stop_PLI_Management;

            UnitMgr.Release (PLI);

            Trans.Step := Trans.Step + 1;

         when 1 =>

            Ret_Value_from_Call := UnitMgr.No_Reconfiguration;
            if Ret_Value_from_Call then

               Ret_Value_from_Call := UnitMgr.State_by_U_Is (
                  GPS, GPS_Fine_Navigation);
               if Ret_Value_from_Call then

                  UnitMgr.Start_Transition_U (GPS, GPS_Coarse_Navigation);

               end if;

               Trans.Step := Trans.Step + 1;

            end if;

         when 2 =>

            Ret_Value_from_Call := UnitMgr.State_by_U_Is (PLI, Off);
            if Ret_Value_from_Call then
               Ret_Value_from_Call := UnitMgr.State_by_U_Is (
                  GPS, GPS_Coarse_Navigation);
               if Ret_Value_from_Call then

                  Trans.Done := True;

               end if;
            end if;

         when others =>

            null;

      end case;

   end Back_To_Nominal;


end ModeMgr.Transitions;
